// doublebuf.v
// David Rowe Feb 11 2009
//
// Serial input, serial output, double buffer
//
// Writing:
//   + data bits are clocked in while wr high
//   + assert wr_finished to latche data to read buffer
//
// Reading:
//   + data bits are clocked out while rd high
//
// Make sure wr_finished isn't asserted at any time while reading 8 
// output bits.
//
// MSB and LSB are swapped between input and output, as RS232 sends LSB
// first, TDM bus sends MSB first

module doublebuf (rst, clk, wr, wr_finished, din, rd, dout);
input        rst;
input        clk;
input        wr;          // assert when writing an 8 bit serial word
input        wr_finished; // assert after 8 bits have been written
input        din;         // serial input data
input        rd;          // assert when reading an 8 bit serial word
output       dout;        // serial output data

parameter DATA_WIDTH = 8;

wire                  dout;
reg [DATA_WIDTH-1:0]  write;         // write register for din
reg [DATA_WIDTH-1:0]  read;          // read register for dout

  always @(posedge clk)
  begin

    if (rst) begin
      write <= 0;
      read  <= 0;
    end

    if (wr) begin
      write <= {write[6:0], din};
    end

    if (rd) begin
      read  <= {1'b0, read[7:1]};
    end

    if (wr_finished) begin
      read <= write;
    end	
  end

  assign dout = read[0];
endmodule
