// sfifo_tb.v
// David Rowe Feb 7 2009
//
// Serial FIFO test bench.

module sfifo_tb;
reg        rst;
reg        clk;
reg        din;
reg        wr;
reg        rd;
wire       dout;
reg  [7:0] testin;
reg  [7:0] testout;
reg  [7:0] data;
reg        wr_finished, rd_finished;

initial begin
  $dumpfile ("sfifo_tb.vcd");
  $dumpvars (1,my_sfifo);

  //$monitor ("clk=%b, wr=%b, din=%b rd=%b dout=%b", clk, wr, din, rd, dout);

  testin = 8'b10110001;
  clk = 0;
  wr = 0;
  wr_finished = 0;
  rd = 0;
  rd_finished = 0;
  clk = 1;
  rst = 1;
  # 1 clk = 0;
  # 1 clk = 1;
  rst = 0; 

  repeat (4) begin
      // write a word

      data = testin;
      wr = 1;
      wr_finished = 0;
      repeat (8) begin
	din = data[7];
	data = {data[6:0], 1'b0};
	#1 clk = 0;
	#1 clk = 1;
      end
      wr = 0;
      wr_finished = 1;
      #1 clk = 0;
      #1 clk = 1;
      wr_finished = 0;

      // read out a word

      testout = 0;
      rd = 1;
      rd_finished = 0;
      repeat (8) begin
	testout = {testout[6:0], dout};
	#1 clk = 0;
	#1 clk = 1;
      end
      rd = 0;
      rd_finished = 1;
      #1 clk = 0;
      #1 clk = 1;
      rd_finished = 0;
    
      $display("testin: %b  testout: %b", testin, testout);
      testin = testin + 1;
  end


end

sfifo my_sfifo(
.rst         (rst),
.clk         (clk),
.wr          (wr),
.wr_finished (wr_finished), 
.din         (din),
.rd          (rd),
.rd_finished (rd_finished), 
.dout        (dout)
);

endmodule

