// transmit.v
// David Rowe 5 Feb 2009
//
// Transmit side processing, top level file.  Converts a 10 bit
// RS232 UART word to an 8 bit TDM bus byte.  Includes FIFO
// buffering to account for router sending faster than we output
// to TDM bus.
//
// Router UART -> transmit.v -> FXS chipset
//                 |        
//                 \-txuart.v
//                 | 
//                 \-doublebuf.v

module transmit (rst, clk, tx, fs, dtx);
input        rst;
input        clk;            // 2.048MHz system clock
input        tx;             // RS232 tx data from router
input        fs;             // TDM bus frame clock
output       dtx;            // TDM bus output to FXS input

wire         wr;
wire         wr_finished;
reg          rd;
reg          rd_finished;
reg  [2:0]   count;
reg          state;

// Send 8 bit TDM byte when FS asserted

parameter IDLE = 1'b0;
parameter SEND = 1'b1;

// state machine iterates on -ve edge as FS is asserted
// just after rising edge of clock

always @(negedge clk)
begin
  if (rst) begin
      state       <= 0;
      rd          <= 0;
      rd_finished <= 0;
  end else begin

    case(state)

    IDLE: if (fs) begin
            state  <= #1 SEND;
            count  <= 0;
            rd     <= 1;
          end
    
    SEND: begin
            count <= count + 1;
            
            if (count == 7) begin
              state <= #1 IDLE;
	      rd_finished <= 1;
              rd          <= 0;
            end
          end

    default: state <= IDLE;

    endcase
  end
end

txuart my_txuart(
.rst         (rst),
.clk         (clk),
.tx          (tx),
.finished    (wr_finished),
.wr          (wr)
);

doublebuf my_doublebuf(
.rst         (rst),
.clk         (clk),
.wr          (wr),
.wr_finished (wr_finished),
.din         (tx),
.rd          (rd),
.dout        (dtx)
);

endmodule
