// txuart.v
// David Rowe 5 Feb 2009
//
// Receives RS232 TX signal from router UART
// Converts it to a synchronous serial bit stream
// Sample tx when wr is asserted to extract serial output data
 
module txuart (rst, clk, tx, finished, wr);
input        rst;
input        clk;            // system clock
input        tx;             // RS232 tx data from router
output       finished;       // asserted after 8th bit
output       wr;             // asserted when each dout bit valid
reg          finished;
reg          wr;

reg    [4:0] clkdiv;         // clock divider register
reg    [3:0] nbits;          // bits received so far
reg          state;
wire         next_state;
reg          prev_tx;        // previous value of tx bit

parameter IDLE   = 1'b0;
parameter SAMPLE = 1'b1;

always @(posedge clk)
begin
  if (rst) begin
      state    <= 0;
      wr       <= 0;
      prev_tx  <= 0;
      finished <= 0;
  end else begin

    case(state)

      IDLE: begin
              finished <= 0;
              if (prev_tx && !tx) begin
                clkdiv <= 9;
                state  <= #1 SAMPLE;
                nbits  <= 0;
              end
            end
      SAMPLE: begin

                clkdiv <= clkdiv + 1;

                if (clkdiv == 17) begin

                  if (nbits == 9) begin
                    state <= #1 IDLE;
		    finished <= 1;
                  end else begin
                    clkdiv <= 0;
                    if (nbits != 0) begin
                      wr   <= 1;
                    end
                    nbits  <= nbits + 1;
                  end

                end else begin
                  wr     <= 0;
                end
              end

      default: state <= IDLE;

    endcase
    prev_tx <= tx;
  end 

end

endmodule
